# Filter\[A: A\]

Take an iterator and a predicate function and return an iterator
that only returns items that match the predicate.

## Example program

Only print command line arguments that are more than 3 characters long.

```pony
use "itertools"

actor Main
  new create(env: Env) =>
    let fn = {(s: String): Bool => x.size() > 3 }
    for x in Filter[String](env.args.slice(1).values(), fn) do
      env.out.print(x)
    end
```


```pony
class ref Filter[A: A] is
  Iterator[A] ref
```

#### Implements

* [Iterator](builtin-Iterator)\[A\] ref

---

## Constructors

### create

```pony
new ref create(
  iter: Iterator[A^] ref,
  fn: {(A!): Bool ?}[A] val)
: Filter[A] ref^
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[A^\] ref
*   fn: {(A!): Bool ?}[A] val

#### Returns

* [Filter](itertools-Filter)\[A\] ref^

---

## Public Functions

### has_next

```pony
fun ref has_next()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### next

```pony
fun ref next()
: A ?
```

#### Returns

* A ?

---

## Private Functions

### _find_next

```pony
fun ref _find_next()
: None val
```

#### Returns

* [None](builtin-None) val

---

