# MapFn\[A: A, B: B\]

Take an iterator and a function and return an iterator where each
item's value is the application of the given function to the value
in the original iterator.

## Example program

Double all of the numbers given as command line arguments.

```pony
use "itertools"

actor Main
  new create(env: Env) =>
    let fn = {(s: String): I32 => try s.i32() * 2 else 0 end }
    for x in MapFn[String, I32](env.args.slice(1).values(), fn) do
      env.out.print(x.string())
    end
```


```pony
class ref MapFn[A: A, B: B] is
  Iterator[B] ref
```

#### Implements

* [Iterator](builtin-Iterator)\[B\] ref

---

## Constructors

### create

```pony
new ref create(
  iter: Iterator[A] ref,
  f: {(A!): B ?}[A, B] box)
: MapFn[A, B] ref^
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[A\] ref
*   f: {(A!): B ?}[A, B] box

#### Returns

* [MapFn](itertools-MapFn)\[A, B\] ref^

---

## Public Functions

### has_next

```pony
fun ref has_next()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### next

```pony
fun ref next()
: B ?
```

#### Returns

* B ?

---

