# Zip2\[A: A, B: B\]

Zip two iterators together so that each call to next() results in the
a tuple with the next value of the first iterator and the next value
of the second iterator. The number of items returned is the minimum of
the number of items returned by the two iterators.

## Example program

Print the following:

```
1 a
2 b
3 c
```

```pony
use "itertools"

actor Main
  new create(env: Env) =>
    var i1: Array[I32] ref = [as I32: 1, 2, 3]
    var i2: Array[String] ref = ["a", "b", "q", "f"]

    for (x, y) in Zip2[I32, String](i1.values(), i2.values()) do
      env.out.print("".add(x.string()).add(" ").add(y))
    end
```


```pony
class ref Zip2[A: A, B: B] is
  Iterator[(A , B)] ref
```

#### Implements

* [Iterator](builtin-Iterator)\[(A , B)\] ref

---

## Constructors

### create

```pony
new ref create(
  i1: Iterator[A] ref,
  i2: Iterator[B] ref)
: Zip2[A, B] ref^
```
#### Parameters

*   i1: [Iterator](builtin-Iterator)\[A\] ref
*   i2: [Iterator](builtin-Iterator)\[B\] ref

#### Returns

* [Zip2](itertools-Zip2)\[A, B\] ref^

---

## Public Functions

### has_next

```pony
fun ref has_next()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### next

```pony
fun ref next()
: (A , B) ?
```

#### Returns

* (A , B) ?

---

