# Zip4\[A: A, B: B, C: C, D: D\]

Zip four iterators together so that each call to next() results in the
a tuple with the next value of each of the iterators. The number of
items returned is the minimum of the number of items returned by the
iterators.

## Example Program

Print the following:

```
1 a 0.5 m
2 b 9.8 n
3 c 3.3 o
```

```pony
use "itertools"

actor Main
  new create(env: Env) =>
    var i1: Array[I32] ref = [as I32: 1, 2, 3]
    var i2: Array[String] ref = ["a", "b", "q", "f"]
    var i3: Array[Float] ref = [as F32: 0.5, 9.8, 3.3]
    var i4: Array[String] ref = ["m", "n", "o", "p", "q", "r"]

    for (x, y, z, xx) in Zip3[I32, String, F32](i1.values(), i2.values(),
      i3.values(), i4.values()) do
      env.out.print("".add(x.string()).add(" ").add(y).add(" ")
        .add(z.string()).add(" ").add(xx.string()))
    end
```


```pony
class ref Zip4[A: A, B: B, C: C, D: D] is
  Iterator[(A , B , C , D)] ref
```

#### Implements

* [Iterator](builtin-Iterator)\[(A , B , C , D)\] ref

---

## Constructors

### create

```pony
new ref create(
  i1: Iterator[A] ref,
  i2: Iterator[B] ref,
  i3: Iterator[C] ref,
  i4: Iterator[D] ref)
: Zip4[A, B, C, D] ref^
```
#### Parameters

*   i1: [Iterator](builtin-Iterator)\[A\] ref
*   i2: [Iterator](builtin-Iterator)\[B\] ref
*   i3: [Iterator](builtin-Iterator)\[C\] ref
*   i4: [Iterator](builtin-Iterator)\[D\] ref

#### Returns

* [Zip4](itertools-Zip4)\[A, B, C, D\] ref^

---

## Public Functions

### has_next

```pony
fun ref has_next()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### next

```pony
fun ref next()
: (A , B , C , D) ?
```

#### Returns

* (A , B , C , D) ?

---

