# Promise\[A: [Any](builtin-Any) #share\]

A promise to eventually produce a result of type A. This promise can either
be fulfilled or rejected.

Any number of promises can be chained after this one.


```pony
actor tag Promise[A: Any #share]
```

## Constructors

### create

```pony
new tag create()
: Promise[A #share] tag^
```

#### Returns

* [Promise](promises-Promise)\[A #share\] tag^

---

## Public Behaviours

### apply

Fulfill the promise.


```pony
be apply(
  value: A #share)
```
#### Parameters

*   value: A #share

---

### reject

Reject the promise.


```pony
be reject()
```

---

## Public Functions

### next\[B: [Any](builtin-Any) #share\]

Chain a promise after this one.

When this promise is fulfilled, the result of type A is passed to the
fulfill function, generating in an intermediate result of type B. This
is then used to fulfill the next promise in the chain.

If there is no fulfill function, or if the fulfill function raises an
error, then the next promise in the chain will be rejected.

If this promise is rejected, this step's reject function is called with no
input, generating an intermediate result of type B which is used to
fulfill the next promise in the chain.

If there is no reject function, of if the reject function raises an error,
then the next promise in the chain will be rejected.


```pony
fun tag next[B: Any #share](
  fulfill: Fulfill[A #share, B #share] iso,
  rejected: Reject[B #share] iso = qualify)
: Promise[B #share] tag
```
#### Parameters

*   fulfill: [Fulfill](promises-Fulfill)\[A #share, B #share\] iso
*   rejected: [Reject](promises-Reject)\[B #share\] iso = qualify

#### Returns

* [Promise](promises-Promise)\[B #share\] tag

---

## Private Behaviours

### _attach

Attaches a step asynchronously. If this promise has already been fulfilled
or rejected, immediately fulfill or reject the incoming step. Otherwise,
keep it in a list.


```pony
be _attach(
  attach: _IThen[A #share] iso)
```
#### Parameters

*   attach: [_IThen](promises-_IThen)\[A #share\] iso

---

