# Random

The `Random` trait should be implemented by all random number generators. The
only method you need to implement is `fun ref next(): 64`. Once that method
has been implemented, the `Random` trait provides default implementations of
conversions to other number types.


```pony
trait ref Random
```

## Public Functions

### has_next

If used as an iterator, this always has another value.


```pony
fun tag has_next()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### next

A random integer in [0, 2^64)


```pony
fun ref next()
: U64 val
```

#### Returns

* [U64](builtin-U64) val

---

### u8

A random integer in [0, 2^8)


```pony
fun ref u8()
: U8 val
```

#### Returns

* [U8](builtin-U8) val

---

### u16

A random integer in [0, 2^16)


```pony
fun ref u16()
: U16 val
```

#### Returns

* [U16](builtin-U16) val

---

### u32

A random integer in [0, 2^32)


```pony
fun ref u32()
: U32 val
```

#### Returns

* [U32](builtin-U32) val

---

### u64

A random integer in [0, 2^64)


```pony
fun ref u64()
: U64 val
```

#### Returns

* [U64](builtin-U64) val

---

### u128

A random integer in [0, 2^128)


```pony
fun ref u128()
: U128 val
```

#### Returns

* [U128](builtin-U128) val

---

### int

A random integer in [0, n)


```pony
fun ref int(
  n: U64 val)
: U64 val
```
#### Parameters

*   n: [U64](builtin-U64) val

#### Returns

* [U64](builtin-U64) val

---

### real

A random number in [0, 1)


```pony
fun ref real()
: F64 val
```

#### Returns

* [F64](builtin-F64) val

---

